unit TESerialDemoForm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ESerial;

type
  TTESerialTest = class(TForm)
    SendButton: TButton;
    LogData: TMemo;
    ClearLogButton: TButton;
    ExitButton: TButton;
    StringsForSending: TMemo;
    PortSelector: TComboBox;
    Label2: TLabel;
    Label1: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    TimeOutOnReceive: TEdit;
    GapBetweenMessages: TEdit;
    BaudRateSelector: TComboBox;
    Label5: TLabel;
    ClearInputButton: TButton;
    SendPack: TButton;
    Label6: TLabel;
    Label7: TLabel;
    SendQueueLines: TLabel;
    SendQueueBytes: TLabel;
    procedure ClearLogButtonClick(Sender: TObject);
    procedure ExitButtonClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure SendButtonClick(Sender: TObject);
    procedure ClearInputButtonClick(Sender: TObject);
    procedure SendPackClick(Sender: TObject);
  private
    { Private declarations }
    Serial : TESerial;
    procedure ProcessError(Sender:TObject);
    procedure DataSent(Sender:TObject);
    procedure DataReceived(Sender:TObject);
    procedure PortSelectorChanged(Sender:TObject);
    procedure BaudSelectorChanged(Sender:TObject);
    procedure TimeOutOnReceiveChanged(Sender:TObject);
    procedure GapBetweenMessagesChanged(Sender:TObject);
  public
    { Public declarations }
  end;

var
  TESerialTest: TTESerialTest;

implementation

{$R *.DFM}

procedure TTESerialTest.ClearLogButtonClick(Sender: TObject);
begin
LogData.Clear;
end;

procedure TTESerialTest.ExitButtonClick(Sender: TObject);
begin
Close;
end;

procedure TTESerialTest.ProcessError;
begin
LogData.Lines.Add('< - Serial Error : '+Serial.FormattedErrorMessage);
end;

procedure TTESerialTest.DataSent;
begin
SendQueueLines.Caption:= '0'; SendQueueBytes.Caption:= '0';
LogData.Lines.Add('---<<<All bytes sent>>>---');
end;

procedure TTESerialTest.DataReceived;
var
  RecData : AnsiString;
begin
SendQueueLines.Caption:= IntToStr(Serial.SendQueueLineCount);
SendQueueBytes.Caption:= IntToStr(Serial.SendQueueByteCount);
repeat
if LogData.Lines.Count>300 then LogData.Clear;
RecData:= Serial.ReceiveData; LogData.Lines.Add('< - Received : '+RecData);
until Serial.ReceiveQueueLineCount=0;
end;

procedure TTESerialTest.FormCreate(Sender: TObject);
var
  I : Byte;
begin
Randomize;
Serial:= TESerial.Create(Self);
with BaudRateSelector do
     for I:= Low(ABaudRates) to High(ABaudRates) do
         begin
         Items.Add(IntToStr(ABaudRates[I]));
         if ABaudRates[I]=DefaultBaudRate then ItemIndex:= I-1;
         end;
GapBetweenMessages.Text:= IntToStr(DefaultMessageGap);
TimeOutOnReceive.Text:= IntToStr(DefaultTimeOutOnRec);
with Serial do
     begin
     for I:= Low(_Ports) to High(_Ports) do
         begin
         Serial.Port:= I; Serial.Active:= true;
         if Active then PortSelector.Items.Add(IntToStr(I));
         end;
     Active:= false; PortSelector.ItemIndex:= 0;
     end;
with Serial do
     begin
     Port:= StrToIntDef(PortSelector.Items[PortSelector.ItemIndex],
                        DefaultPortNumber);
     OnError:= ProcessError; OnDataSent:= DataSent;
     OnDataReceived:= DataReceived;
     Active:= true;
     end;
PortSelector.OnChange:= PortSelectorChanged;
BaudRateSelector.OnChange:= BaudSelectorChanged;
TimeOutOnReceive.OnChange:= TimeOutOnReceiveChanged;
GapBetweenMessages.OnChange:= GapBetweenMessagesChanged;
end;

procedure TTESerialTest.FormDestroy(Sender: TObject);
begin
Serial.Free;
end;

procedure TTESerialTest.SendButtonClick(Sender: TObject);
var
  I : Word;
begin
if StringsForSending.Lines.Count=0 then begin
                                        ShowMessage('Nothing to send'); Exit;
                                        end;
SendButton.Enabled:= false;
with StringsForSending do
     for I:= 0 to Lines.Count-1 do
         begin
         Serial.SendData(Lines[I]); //it does not put if port is not active
         SendQueueLines.Caption:= IntToStr(Serial.SendQueueLineCount);
         SendQueueBytes.Caption:= IntToStr(Serial.SendQueueByteCount);
         if Serial.Active
            then LogData.Lines.Add('- > Placed in the queue : "'+Lines[I]+'"');
         end;
SendButton.Enabled:= true;
end;

procedure TTESerialTest.ClearInputButtonClick(Sender: TObject);
begin
StringsForSending.Clear;
end;

procedure TTESerialTest.PortSelectorChanged(Sender:TObject);
begin
with Serial do
     begin
     Port:= StrToInt(PortSelector.Items[PortSelector.ItemIndex]);
     Active:= true;
     if not(Active) then LogData.Lines.Add('COM PORT COULD NOT BE OPENED');
     end;
end;

procedure TTESerialTest.BaudSelectorChanged(Sender:TObject);
begin
with Serial do
     begin
     BaudRate:= StrToInt(BaudRateSelector.Items[BaudRateSelector.ItemIndex]);
     Active:= true;
     if not(Active) then LogData.Lines.Add('COM PORT COULD NOT BE OPENED');
     end;
end;

procedure TTESerialTest.TimeOutOnReceiveChanged(Sender:TObject);
begin
TimeOutOnReceive.ReadOnly:= true;
with Serial do
     begin
     TimeOutOnRec:= StrToIntDef(TimeOutOnReceive.Text,0);
     TimeOutOnReceive.Text:= IntToStr(TimeOutOnRec);
     Active:= true;
     if not(Active) then LogData.Lines.Add('COM PORT COULD NOT BE OPENED');
     end;
TimeOutOnReceive.ReadOnly:= false;
end;

procedure TTESerialTest.GapBetweenMessagesChanged(Sender:TObject);
begin
GapBetweenMessages.ReadOnly:= true;
with Serial do
     begin
     MessageGap:= StrToIntDef(GapBetweenMessages.Text,0);;
     GapBetweenMessages.Text:= IntToStr(MessageGap);
     Active:= true;
     if not(Active) then LogData.Lines.Add('COM PORT COULD NOT BE OPENED');
     end;
GapBetweenMessages.ReadOnly:= false;
end;

procedure TTESerialTest.SendPackClick(Sender: TObject);
var
  I,J : Word;
  Str : String;
begin
SendPack.Enabled:= false;
with StringsForSending do
     for I:= 0 to 30 do
         begin
         Str:= '';
         for J:= 1 to 200 do Str:= Str+Chr(Random(10)+$39);
         Serial.SendData(IntToStr(I)+' : '+Str);
         SendQueueLines.Caption:= IntToStr(Serial.SendQueueLineCount);
         SendQueueBytes.Caption:= IntToStr(Serial.SendQueueByteCount);
         end;
SendPack.Enabled:= true;
end;

end.

